#######################################################################
#                         Experiment backends                         #
#######################################################################

abstract type AbstractExpBackend end
mutable struct LocalExpBackend <: AbstractExpBackend end
CURRENT_EXPBACKEND = LocalExpBackend()
expbackend() = CURRENT_EXPBACKEND

run_on_backend(fun, fun_args=[], fun_kwargs=Dict(); backend_kwargs...) = 
	_run_on_backend(CURRENT_EXPBACKEND, fun, fun_args, fun_kwargs; backend_kwargs...)
update_backend(e) = _update_backend(CURRENT_EXPBACKEND, e)
get_results(args...) = _get_results(CURRENT_EXPBACKEND, args...)
resultsbasedir() = _resultsbasedir(CURRENT_EXPBACKEND)

# Defaults
function Base.finalize(x::AbstractExpBackend) end
function _update_backend(x::AbstractExpBackend, args...) end

# ===== Local Backend =====

	# lines_computed = nbcomputedlines(b.c, logdictpath, ue.basename)
function nbcomputedlines(path, basename)
	r = joinpath(path, basename)
	# Sum numbers of lines of all files path/basename* (-1 line of header for each file)
	# Return 0 if there are no such files
	strcmd = `sh -c "if ls $r* 1> /dev/null 2>&1; then sed -s 1d $r* | wc -l; else echo 0; fi"`
	parse(Int, read(strcmd, String))
end

function _run_on_backend(::LocalExpBackend, fun, fun_args, fun_kwargs; kwargs...)
	if ~isempty(kwargs)
		@warn "Ignoring following kwargs: $(join([kw[1] for kw in kwargs], ", ")) (not supported by LocalExpBackend)."
	end
	ue = fun_args[1]
	jobname = "$(ue.prms.name)__$(ue.basename)"[1:min(end,98)]	# Just for security
	logdictpath = joinpath(ue.prms.resultsdir, "$(ue.prms.logdict.name)")
	println(logdictpath)
	println(ue.basename)
	lines_computed = nbcomputedlines(logdictpath, ue.basename)
	rem_iterations = ue.prms.iterations - lines_computed
	if rem_iterations == 0
		println("Results already present for $jobname, ignoring.") 
	elseif rem_iterations > 0
		println("Running $jobname (remaining iterations: $(rem_iterations)/$(ue.prms.iterations))…") 
		fun(fun_args...; 
			remaining_iterations = rem_iterations, 
			fun_kwargs...)
	else
		@warn("More observations than expected. This should not happen.")
	end
end
# _get_results(::LocalExpBackend, s) = error("Not implemented yet.") # TODO
_resultsbasedir(::LocalExpBackend) = joinpath(pwd(), "results_LocalExpBE")

function runlocally(force = false)
	global CURRENT_EXPBACKEND
	if !(typeof(CURRENT_EXPBACKEND) <: LocalExpBackend) || force
		finalize(CURRENT_EXPBACKEND)
		CURRENT_EXPBACKEND = LocalExpBackend()
	end
end

local_results_at(dir, s) = local_results(s; resdir=joinpath(dir, "results_LocalExpBE"))
function local_results(s; resdir = nothing)
	if resdir isa Nothing 
		resdir = joinpath(pwd(), "results_LocalExpBE")
	end
	dir = joinpath(resdir, "results_$s", "logdict")
	t = DataFrame()
	for f in readdir(dir)
		# nf = CSV.read(joinpath(dir,f))
		nf = CSV.read(joinpath(dir,f), DataFrame)
		t = vcat_addmissing(t, nf)
	end
	return t
end

isrunninglocally() = CURRENT_EXPBACKEND isa LocalExpBackend
